@@ -49,7 +49,9 @@ class AwlSimMessage(object):
 	MSG_ID_EXCEPTION	= enum.item
 	MSG_ID_PING		= enum.item
 	MSG_ID_PONG		= enum.item
+	MSG_ID_RESET		= enum.item
 	MSG_ID_RUNSTATE		= enum.item
+	MSG_ID_LOAD_SYMTAB	= enum.item
 	MSG_ID_LOAD_CODE	= enum.item
 	MSG_ID_LOAD_HW		= enum.item
 	MSG_ID_SET_OPT		= enum.item
@@ -140,6 +142,10 @@ class AwlSimMessage_PONG(AwlSimMessage):
 	def __init__(self):
 		AwlSimMessage.__init__(self, AwlSimMessage.MSG_ID_PONG)
 
+class AwlSimMessage_RESET(AwlSimMessage):
+	def __init__(self):
+		AwlSimMessage.__init__(self, AwlSimMessage.MSG_ID_RESET)
+
 class AwlSimMessage_RUNSTATE(AwlSimMessage):
 	enum.start
 	STATE_STOP	= enum.item
@@ -184,6 +190,26 @@ class AwlSimMessage_EXCEPTION(AwlSimMessage):
 			raise TransferError("EXCEPTION: Unicode error")
 		return cls(text)
 
+class AwlSimMessage_LOAD_SYMTAB(AwlSimMessage):
+	def __init__(self, symTabText):
+		AwlSimMessage.__init__(self, AwlSimMessage.MSG_ID_LOAD_SYMTAB)
+		self.symTabText = symTabText
+
+	def toBytes(self):
+		try:
+			data = self.symTabText.encode()
+			return AwlSimMessage.toBytes(self, len(data)) + data
+		except UnicodeError:
+			raise TransferError("LOAD_SYMTAB: Unicode error")
+
+	@classmethod
+	def fromBytes(cls, payload):
+		try:
+			symTabText = payload.decode()
+		except UnicodeError:
+			raise TransferError("LOAD_SYMTAB: Unicode error")
+		return cls(symTabText)
+
 class AwlSimMessage_LOAD_CODE(AwlSimMessage):
 	def __init__(self, code):
 		AwlSimMessage.__init__(self, AwlSimMessage.MSG_ID_LOAD_CODE)
@@ -475,7 +501,9 @@ class AwlSimMessageTransceiver(object):
 		AwlSimMessage.MSG_ID_EXCEPTION		: AwlSimMessage_EXCEPTION,
 		AwlSimMessage.MSG_ID_PING		: AwlSimMessage_PING,
 		AwlSimMessage.MSG_ID_PONG		: AwlSimMessage_PONG,
+		AwlSimMessage.MSG_ID_RESET		: AwlSimMessage_RESET,
 		AwlSimMessage.MSG_ID_RUNSTATE		: AwlSimMessage_RUNSTATE,
+		AwlSimMessage.MSG_ID_LOAD_SYMTAB	: AwlSimMessage_LOAD_SYMTAB,
 		AwlSimMessage.MSG_ID_LOAD_CODE		: AwlSimMessage_LOAD_CODE,
 		AwlSimMessage.MSG_ID_LOAD_HW		: AwlSimMessage_LOAD_HW,
 		AwlSimMessage.MSG_ID_SET_OPT		: AwlSimMessage_SET_OPT,
