@@ -25,6 +25,7 @@ import random
 
 from awlsim.cpuspecs import *
 from awlsim.parser import *
+from awlsim.symbolparser import *
 from awlsim.datatypes import *
 from awlsim.instructions.all_insns import *
 from awlsim.operators import *
@@ -118,6 +119,7 @@ class S7CPU(object):
 			S7CPUSpecs.MNEMONICS_EN		: 0,
 			S7CPUSpecs.MNEMONICS_DE		: 0,
 		}
+		detected = None
 		for mnemonics in (S7CPUSpecs.MNEMONICS_EN,
 				  S7CPUSpecs.MNEMONICS_DE):
 			for block in codeBlocks:
@@ -128,18 +130,26 @@ class S7CPU(object):
 						errorCounts[mnemonics] += 1
 					try:
 						optrans = AwlOpTranslator(None, mnemonics)
-						optrans.translateFrom(rawInsn)
+						optrans.translateFromRawInsn(rawInsn)
 					except AwlSimError:
 						errorCounts[mnemonics] += 1
 			if errorCounts[mnemonics] == 0:
 				# No error. Use these mnemonics.
-				specs.setDetectedMnemonics(mnemonics)
-				return
-		# Select the mnemonics with the lower error count.
-		if errorCounts[S7CPUSpecs.MNEMONICS_EN] <= errorCounts[S7CPUSpecs.MNEMONICS_DE]:
-			specs.setDetectedMnemonics(S7CPUSpecs.MNEMONICS_EN)
-		else:
-			specs.setDetectedMnemonics(S7CPUSpecs.MNEMONICS_DE)
+				detected = mnemonics
+		if detected is None:
+			# Select the mnemonics with the lower error count.
+			if errorCounts[S7CPUSpecs.MNEMONICS_EN] <= errorCounts[S7CPUSpecs.MNEMONICS_DE]:
+				detected = S7CPUSpecs.MNEMONICS_EN
+			else:
+				detected = S7CPUSpecs.MNEMONICS_DE
+		if specs.getMnemonics() != S7CPUSpecs.MNEMONICS_AUTO:
+			# Autodetected mnemonics were already set before
+			if specs.getMnemonics() != detected:
+				raise AwlSimError("Cannot mix multiple AWL files with "\
+					"distinct mnemonics. This error may be caused by "\
+					"incorrect autodetection. "\
+					"Force mnemonics to EN or DE to avoid this error.")
+		specs.setDetectedMnemonics(detected)
 
 	def __translateInsn(self, rawInsn, ip):
 		ex = None
@@ -273,8 +283,32 @@ class S7CPU(object):
 		db.allocate()
 		return db
 
+	# Translate classic symbols ("abc")
+	def __resolveClassicSym(self, block, insn, oper):
+		if oper.type == AwlOperator.SYMBOLIC:
+			symbol = self.symbolTable.findByName(oper.value)
+			if not symbol:
+				raise AwlSimError("Symbol \"%s\" not found in "
+					"symbol table." % oper.value,
+					insn = insn)
+			oper = symbol.operator
+		return oper
+
+	# Translate symbolic OB/FB/FC/DB block name
+	def __resolveBlockName(self, blockTypeId, blockName):
+		if isString(blockName):
+			symbol = self.symbolTable.findByName(blockName)
+			if not symbol:
+				raise AwlSimError("Symbolic block name \"%s\" "
+					"not found in symbol table." % blockName)
+			if symbol.type.type != blockTypeId:
+				raise AwlSimError("Symbolic block name \"%s\" "
+					"has an invalid type." % blockName)
+			return symbol.operator.value.byteOffset
+		return blockName
+
 	# Translate local symbols (#abc)
-	def __resolveNamedLocalSym(self, block, oper):
+	def __resolveNamedLocalSym(self, block, insn, oper):
 		if oper.type == AwlOperator.NAMED_LOCAL:
 			newOper = block.interface.getOperatorForFieldName(oper.value, False)
 			newOper.setInsn(oper.insn)
@@ -282,7 +316,7 @@ class S7CPU(object):
 		return oper
 
 	# Translate local symbol pointers (P##abc)
-	def __resolveNamedLocalPointer(self, block, oper):
+	def __resolveNamedLocalPointer(self, block, insn, oper):
 		if oper.type == AwlOperator.NAMED_LOCAL_PTR:
 			newOper = block.interface.getOperatorForFieldName(oper.value, True)
 			newOper.setInsn(oper.insn)
@@ -292,23 +326,33 @@ class S7CPU(object):
 	def __resolveSymbols_block(self, block):
 		for insn in block.insns:
 			for i in range(len(insn.ops)):
+				insn.ops[i] = self.__resolveClassicSym(block,
+								insn, insn.ops[i])
 				insn.ops[i] = self.__resolveNamedLocalSym(block,
-								insn.ops[i])
+								insn, insn.ops[i])
 				if insn.ops[i].type == AwlOperator.INDIRECT:
+					insn.ops[i].offsetOper = \
+						self.__resolveClassicSym(block,
+								insn, insn.ops[i].offsetOper)
 					insn.ops[i].offsetOper = \
 						self.__resolveNamedLocalSym(block,
-								insn.ops[i].offsetOper)
+								insn, insn.ops[i].offsetOper)
 				insn.ops[i] = self.__resolveNamedLocalPointer(block,
-								insn.ops[i])
+								insn, insn.ops[i])
 			for i in range(len(insn.params)):
+				insn.params[i].rvalueOp = self.__resolveClassicSym(block,
+								insn, insn.params[i].rvalueOp)
 				insn.params[i].rvalueOp = self.__resolveNamedLocalSym(block,
-								insn.params[i].rvalueOp)
+								insn, insn.params[i].rvalueOp)
 				if insn.params[i].rvalueOp.type == AwlOperator.INDIRECT:
+					insn.params[i].rvalueOp.offsetOper =\
+						self.__resolveClassicSym(block,
+								insn, insn.params[i].rvalueOp.offsetOper)
 					insn.params[i].rvalueOp.offsetOper =\
 						self.__resolveNamedLocalSym(block,
-								insn.params[i].rvalueOp.offsetOper)
+								insn, insn.params[i].rvalueOp.offsetOper)
 				insn.params[i].rvalueOp = self.__resolveNamedLocalPointer(block,
-								insn.params[i].rvalueOp)
+								insn, insn.params[i].rvalueOp)
 
 	def __resolveSymbols(self):
 		for ob in self.obs.values():
@@ -340,11 +384,15 @@ class S7CPU(object):
 	def load(self, parseTree):
 		# Mnemonics autodetection
 		self.__detectMnemonics(parseTree)
-		# Reset the CPU
-		self.reset()
 		# Translate OBs
-		for obNumber in parseTree.obs.keys():
-			ob = self.__translateCodeBlock(parseTree.obs[obNumber], OB)
+		for obNumber, rawOB in parseTree.obs.items():
+			obNumber = self.__resolveBlockName(AwlDataType.TYPE_OB_X,
+							   obNumber)
+			if obNumber in self.obs:
+				raise AwlSimError("Multiple definitions of "\
+					"OB %d" % obNumber)
+			rawOB.index = obNumber
+			ob = self.__translateCodeBlock(rawOB, OB)
 			self.obs[obNumber] = ob
 			# Create the TEMP-preset handler table
 			try:
@@ -353,39 +401,60 @@ class S7CPU(object):
 				presetHandlerClass = OBTempPresets_dummy
 			self.obTempPresetHandlers[obNumber] = presetHandlerClass(self)
 		# Translate FBs
-		for fbNumber in parseTree.fbs.keys():
-			fb = self.__translateCodeBlock(parseTree.fbs[fbNumber], FB)
+		for fbNumber, rawFB in parseTree.fbs.items():
+			fbNumber = self.__resolveBlockName(AwlDataType.TYPE_FB_X,
+							   fbNumber)
+			if fbNumber in self.fbs:
+				raise AwlSimError("Multiple definitions of "\
+					"FB %d" % fbNumber)
+			rawFB.index = fbNumber
+			fb = self.__translateCodeBlock(rawFB, FB)
 			self.fbs[fbNumber] = fb
 		# Translate FCs
-		for fcNumber in parseTree.fcs.keys():
-			fc = self.__translateCodeBlock(parseTree.fcs[fcNumber], FC)
+		for fcNumber, rawFC in parseTree.fcs.items():
+			fcNumber = self.__resolveBlockName(AwlDataType.TYPE_FC_X,
+							   fcNumber)
+			if fcNumber in self.fcs:
+				raise AwlSimError("Multiple definitions of "\
+					"FC %d" % fcNumber)
+			rawFC.index = fcNumber
+			fc = self.__translateCodeBlock(rawFC, FC)
 			self.fcs[fcNumber] = fc
 			bounceDB = self.__allocateFCBounceDB(fc, False)
 			self.dbs[bounceDB.index] = bounceDB
-		# Create the SFB tables
-		for sfbNumber in SFB_table.keys():
-			if sfbNumber < 0 and not self.__extendedInsnsEnabled:
-				continue
-			sfb = SFB_table[sfbNumber](self)
-			sfb.interface.buildDataStructure()
-			self.sfbs[sfbNumber] = sfb
-		# Create the SFC tables
-		for sfcNumber in SFC_table.keys():
-			if sfcNumber < 0 and not self.__extendedInsnsEnabled:
-				continue
-			sfc = SFC_table[sfcNumber](self)
-			sfc.interface.buildDataStructure()
-			self.sfcs[sfcNumber] = sfc
-			bounceDB = self.__allocateFCBounceDB(sfc, True)
-			self.dbs[bounceDB.index] = bounceDB
+
+		if not self.sfbs:
+			# Create the SFB tables
+			for sfbNumber in SFB_table.keys():
+				if sfbNumber < 0 and not self.__extendedInsnsEnabled:
+					continue
+				sfb = SFB_table[sfbNumber](self)
+				sfb.interface.buildDataStructure()
+				self.sfbs[sfbNumber] = sfb
+		if not self.sfcs:
+			# Create the SFC tables
+			for sfcNumber in SFC_table.keys():
+				if sfcNumber < 0 and not self.__extendedInsnsEnabled:
+					continue
+				sfc = SFC_table[sfcNumber](self)
+				sfc.interface.buildDataStructure()
+				self.sfcs[sfcNumber] = sfc
+				bounceDB = self.__allocateFCBounceDB(sfc, True)
+				self.dbs[bounceDB.index] = bounceDB
+
 		# Translate DBs
-		for dbNumber in parseTree.dbs.keys():
-			db = self.__translateDB(parseTree.dbs[dbNumber])
+		for dbNumber, rawDB in parseTree.dbs.items():
+			dbNumber = self.__resolveBlockName(AwlDataType.TYPE_DB_X,
+							   dbNumber)
+			if dbNumber in self.dbs:
+				raise AwlSimError("Multiple definitions of "\
+					"DB %d" % dbNumber)
+			rawDB.index = dbNumber
+			db = self.__translateDB(rawDB)
 			self.dbs[dbNumber] = db
-		# Resolve symbolic instructions and operators
-		self.__resolveSymbols()
-		# Run some static sanity checks on the code
-		self.__staticSanityChecks()
+
+	def loadSymbolTable(self, symbolTable):
+		self.symbolTable.merge(symbolTable)
 
 	def reallocate(self, force=False):
 		if force or (self.specs.nrAccus == 4) != self.is4accu:
@@ -433,6 +502,7 @@ class S7CPU(object):
 		self.sfbs = {
 			# System SFBs
 		}
+		self.symbolTable = SymbolTable()
 		self.reallocate(force=True)
 		self.ar1 = Adressregister()
 		self.ar2 = Adressregister()
@@ -539,6 +609,11 @@ class S7CPU(object):
 
 	# Run startup code
 	def startup(self):
+		# Resolve symbolic instructions and operators
+		self.__resolveSymbols()
+		# Run some static sanity checks on the code
+		self.__staticSanityChecks()
+
 		self.updateTimestamp()
 		self.__speedMeasureStartTime = self.now
 		self.__speedMeasureStartInsnCount = 0
@@ -828,6 +903,18 @@ class S7CPU(object):
 	def fetchIMM(self, operator):
 		return operator.value
 
+	def fetchDBLG(self, operator):
+		return self.dbRegister.struct.getSize()
+
+	def fetchDBNO(self, operator):
+		return self.dbRegister.index
+
+	def fetchDILG(self, operator):
+		return self.diRegister.struct.getSize()
+
+	def fetchDINO(self, operator):
+		return self.diRegister.index
+
 	def fetchSTW(self, operator):
 		if operator.width == 1:
 			return self.statusWord.getByBitNumber(operator.value.bitOffset)
@@ -967,6 +1054,10 @@ class S7CPU(object):
 		AwlOperator.MEM_T		: fetchT,
 		AwlOperator.MEM_Z		: fetchZ,
 		AwlOperator.MEM_PE		: fetchPE,
+		AwlOperator.MEM_DBLG		: fetchDBLG,
+		AwlOperator.MEM_DBNO		: fetchDBNO,
+		AwlOperator.MEM_DILG		: fetchDILG,
+		AwlOperator.MEM_DINO		: fetchDINO,
 		AwlOperator.MEM_STW		: fetchSTW,
 		AwlOperator.MEM_STW_Z		: fetchSTW_Z,
 		AwlOperator.MEM_STW_NZ		: fetchSTW_NZ,
